@@ -162,13 +162,13 @@ static unsigned int IsSUN(const unsigned char *magick,const size_t length)
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(const unsigned char *compressed_pixels,
+%      MagickPassFail DecodeImage(const unsigned char *compressed_pixels,
 %        const size_t length,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the pixels are
-%      uncompressed without error, otherwise False.
+%    o status:  Method DecodeImage returns True (MagickPass) if all the
+%      pixels are uncompressed without error, otherwise False (MagickFail).
 %
 %    o compressed_pixels:  The address of a byte (8 bits) array of compressed
 %      pixel data.
@@ -209,26 +209,37 @@ DecodeImage(const unsigned char *compressed_pixels,
   q=pixels;
   while (((size_t) (p-compressed_pixels) < compressed_size) &&
          ((size_t) (q-pixels) < pixels_size))
-  {
-    byte=(*p++);
-    if (byte != 128U)
-      *q++=byte;
-    else
-      {
-        /*
-          Runlength-encoded packet: <count><byte>
-        */
-        count=(*p++);
-        if (count > 0)
-          byte=(*p++);
-        while ((count >= 0) && ((size_t) (q-pixels) < pixels_size))
+    {
+      byte=(*p++);
+      if (byte != 128U)
         {
+          /*
+            Stand-alone byte
+          */
           *q++=byte;
-          count--;
         }
-     }
-  }
-  return (((size_t) (q-pixels) == pixels_size) ? MagickTrue : MagickFalse);
+      else
+        {
+          /*
+            Runlength-encoded packet: <count><byte>
+          */
+          if (((size_t) (p-compressed_pixels) >= compressed_size))
+            break;
+          count=(*p++);
+          if (count > 0)
+            {
+              if (((size_t) (p-compressed_pixels) >= compressed_size))
+                break;
+              byte=(*p++);
+            }
+          while ((count >= 0) && ((size_t) (q-pixels) < pixels_size))
+            {
+              *q++=byte;
+              count--;
+            }
+        }
+    }
+  return (((size_t) (q-pixels) == pixels_size) ? MagickPass : MagickFail);
 }
 
 /*
@@ -376,6 +387,9 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
     image->columns=sun_info.width;
     image->rows=sun_info.height;
+    if (((unsigned long) ((long) image->columns) != image->columns) ||
+        ((unsigned long) ((long) image->rows) != image->rows))
+      ThrowReaderException(CoderError,ImageColumnOrRowSizeIsNotSupported,image);
     if (CheckImagePixelLimits(image, exception) != MagickPass)
         ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
     image->depth=sun_info.depth <= 8 ? 8 : QuantumDepth;
@@ -604,6 +618,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (x=0; x < (long) image->columns; x++)
               {
                 index=(*p++);
+                VerifyColormapIndex(image,index);
                 indexes[x]=index;
                 q[x]=image->colormap[index];
               }
@@ -923,15 +938,19 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           *q;
 
         size_t
-          length;
+          length,
+          pad;
 
         unsigned char
           *pixels;
 
         /*
           Allocate memory for pixels.
+
+          Scanlines are padded to 16-bit boundary so account for padding.
         */
-        length=image->columns*sizeof(PixelPacket);
+        pad=(image->columns & 0x01 ? 1 : 0);
+        length=(image->columns + pad) *sizeof(PixelPacket);
         pixels=MagickAllocateMemory(unsigned char *,length);
         if (pixels == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
