@@ -315,7 +315,7 @@ static void draw_pattern_push( wmfAPI* API,
                                unsigned long rows )
 {
   char
-    pattern_id[30];
+    pattern_id[MaxTextExtent];
 
   FormatString(pattern_id,"brush_%lu",id);
   DrawPushPattern(WmfDrawContext,pattern_id,0,0,columns,rows);
@@ -575,8 +575,9 @@ static void ipa_bmp_read(wmfAPI * API, wmfBMP_Read_t * bmp_read) {
     }
 }
 
-static void ipa_bmp_free(wmfAPI * ARGUNUSED(API), wmfBMP * bmp)
+static void ipa_bmp_free(wmfAPI * API, wmfBMP * bmp)
 {
+  ARG_NOT_USED(API);
   DestroyImageList((Image*)bmp->data);
   bmp->data = (void*) 0;
   bmp->width = (U16) 0;
@@ -673,7 +674,8 @@ static void ipa_device_begin(wmfAPI * API)
       GetExceptionInfo(&exception);
 
       image_info = CloneImageInfo((ImageInfo *) 0);
-      (void) strcpy(image_info->filename, ddata->image_info->texture);
+      (void) strlcpy(image_info->filename, ddata->image_info->texture,
+                     sizeof(image_info->filename));
 
       if ( ddata->image_info->size )
         (void) CloneString(&image_info->size,ddata->image_info->size);
@@ -683,9 +685,9 @@ static void ipa_device_begin(wmfAPI * API)
       if(image)
         {
           char
-            pattern_id[30];
+            pattern_id[MaxTextExtent];
 
-          (void) strcpy(image->magick,"MIFF");
+          (void) strlcpy(image->magick,"MIFF",sizeof(image->magick));
           DrawPushDefs(WmfDrawContext);
           draw_pattern_push(API, ddata->pattern_id, image->columns, image->rows);
           DrawComposite(WmfDrawContext, CopyCompositeOp, 0, 0, image->columns, image->rows, image);
@@ -1139,7 +1141,7 @@ static void ipa_region_clip(wmfAPI *API, wmfPolyRectangle_t *poly_rect)
   if(poly_rect->count > 0)
     {
       char
-        clip_path_id[30];
+        clip_path_id[MaxTextExtent];
 
       /* Define clip path */
       ddata->clip_path_id++;
@@ -1480,26 +1482,34 @@ static void ipa_draw_text(wmfAPI * API, wmfDrawText_t * draw_text)
 
 }
 
-static void ipa_udata_init(wmfAPI * ARGUNUSED(API), wmfUserData_t * ARGUNUSED(userdata))
+static void ipa_udata_init(wmfAPI *API, wmfUserData_t *userdata)
 {
+  ARG_NOT_USED(API);
+  ARG_NOT_USED(userdata);
   /* wmf_magick_t* ddata = WMF_MAGICK_GetData (API); */
 
 }
 
-static void ipa_udata_copy(wmfAPI * ARGUNUSED(API), wmfUserData_t * ARGUNUSED(userdata))
+static void ipa_udata_copy(wmfAPI *API, wmfUserData_t *userdata)
 {
+  ARG_NOT_USED(API);
+  ARG_NOT_USED(userdata);
   /* wmf_magick_t* ddata = WMF_MAGICK_GetData (API); */
 
 }
 
-static void ipa_udata_set(wmfAPI * ARGUNUSED(API), wmfUserData_t * ARGUNUSED(userdata))
+static void ipa_udata_set(wmfAPI *API, wmfUserData_t *userdata)
 {
+  ARG_NOT_USED(API);
+  ARG_NOT_USED(userdata);
   /* wmf_magick_t* ddata = WMF_MAGICK_GetData (API); */
 
 }
 
-static void ipa_udata_free(wmfAPI * ARGUNUSED(API), wmfUserData_t * ARGUNUSED(userdata))
+static void ipa_udata_free(wmfAPI *API, wmfUserData_t *userdata)
 {
+  ARG_NOT_USED(API);
+  ARG_NOT_USED(userdata);
   /* wmf_magick_t* ddata = WMF_MAGICK_GetData (API); */
 
 }
@@ -1616,7 +1626,7 @@ static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_appl
         DrawPopDefs(WmfDrawContext);
         {
           char
-            pattern_id[30];
+            pattern_id[MaxTextExtent];
 
           FormatString(pattern_id, "#brush_%lu", ddata->pattern_id);
 
@@ -1727,7 +1737,7 @@ static void util_set_brush(wmfAPI * API, wmfDC * dc, const BrushApply brush_appl
 
             {
               char
-                pattern_id[30];
+                pattern_id[MaxTextExtent];
 
               FormatString(pattern_id, "#brush_%lu", ddata->pattern_id);
 
@@ -2258,7 +2268,7 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
         {
           if(LocaleCompare(wmf_font_name, SubFontMap[i].name) == 0)
             {
-              (void) strcpy(target,SubFontMap[i].mapping);
+              (void) strlcpy(target,SubFontMap[i].mapping,sizeof(target));
               break;
             }
         }
@@ -2290,11 +2300,12 @@ static void lite_font_map( wmfAPI* API, wmfFont* font)
 }
 
 /* Initialize API font structures */
-static void lite_font_init( wmfAPI* API, wmfAPI_Options* ARGUNUSED(options))
+static void lite_font_init( wmfAPI* API, wmfAPI_Options* options)
 {
   wmfFontData
     *font_data;
 
+  ARG_NOT_USED(options);
   API->fonts = 0;
 
   /* Allocate wmfFontData data structure */
