@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2015 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -21,7 +21,7 @@
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
-%                              June 2000 - 2007                               %
+%                              June 2000 - 2015                               %
 %                         Rework for GraphicsMagick                           %
 %                              Bob Friesenhahn                                %
 %                               Feb-May 2003                                  %
@@ -280,49 +280,48 @@ static MagickPassFail InsertRow(unsigned char *p,long y, Image *image, int bpp)
     case 2:  /* Convert PseudoColor scanline. */
       {
         indexes=AccessMutableIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        x = 0;
+        while(x < (long)image->columns-3)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
+            index = (IndexPacket)((*p >> 6) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 4) & 0x3);
+            index = (IndexPacket)((*p >> 4) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 2) & 0x3);
+            index = (IndexPacket)((*p >> 2) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[index];
-            index=(IndexPacket) ((*p) & 0x3);
+            indexes[x++]=index;
+            *q++ = image->colormap[index];
+            index = (IndexPacket)((*p) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x+1]=index;
-            *q++=image->colormap[index];
+            indexes[x++]=index;
+            *q++ = image->colormap[index];
             p++;
-          }
-        if ((image->columns % 4) != 0)
+		  }
+        if(x < (long) image->columns)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
+            index = (IndexPacket) ((*p >> 6) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++] = index;
             *q++=image->colormap[index];
-            if ((image->columns % 4) >= 1)
-
+            if(x < (long) image->columns)
               {
-                index=(IndexPacket) ((*p >> 4) & 0x3);
+                index = (IndexPacket) ((*p >> 4) & 0x3);
                 VerifyColormapIndex(image,index);
-                indexes[x]=index;
+                indexes[x++] = index;
                 *q++=image->colormap[index];
-                if ((image->columns % 4) >= 2)
-
+                if(x < (long) image->columns)
                   {
-                    index=(IndexPacket) ((*p >> 2) & 0x3);
+                    index = (IndexPacket)((*p >> 2) & 0x3);
                     VerifyColormapIndex(image,index);
-                    indexes[x]=index;
+                    indexes[x] = index;
                     *q++=image->colormap[index];
                   }
               }
-            p++;
+            /* p++; */
           }       
         RetVal = MagickPass;
         break;
@@ -383,12 +382,22 @@ static int UnpackWPGRaster(Image *image,int bpp)
   y=0;
 
   ldblk=(long) ((bpp*image->columns+7)/8);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Raster allocation size: %ld byte%s",
+                        ldblk, (ldblk > 1 ? "s" : ""));
   BImgBuff=MagickAllocateMemory(unsigned char *,(size_t) ldblk);
   if(BImgBuff==NULL) return(-2);
+  (void) memset(BImgBuff,0,(size_t) ldblk);
 
   while(y<(long) image->rows)
     {
-      bbuf=ReadBlobByte(image);
+      i = ReadBlobByte(image);
+	  if(i==EOF)
+	    {
+          MagickFreeMemory(BImgBuff);
+          return(-5);
+        }
+      bbuf = i;
 
       RunCount=bbuf & 0x7F;
       if(bbuf & 0x80)
@@ -459,17 +468,24 @@ static int UnpackWPGRaster(Image *image,int bpp)
    } \
 }
 
+#define FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff) \
+  do \
+    { \
+      MagickFreeMemory(BImgBuff); \
+      MagickFreeMemory(UpImgBuff); \
+    } while(0);
+
 /* WPG2 raster reader. */
-static int UnpackWPG2Raster(Image *image,int bpp)
+static int UnpackWPG2Raster(Image *image, int bpp)
 {
   unsigned int
     SampleSize=1;
 
   unsigned char
     bbuf,
-    *BImgBuff,		/* Buffer for a current line. */
-    *UpImgBuff,		/* Buffer for previous line. */
-    *tmpImgBuff,
+    *BImgBuff = (unsigned char *) NULL,	  /* Buffer for a current line. */
+    *UpImgBuff = (unsigned char *) NULL,  /* Buffer for previous line. */
+    *tmpImgBuff = (unsigned char *) NULL,
     RunCount,
     SampleBuffer[8];
 
@@ -484,6 +500,7 @@ static int UnpackWPG2Raster(Image *image,int bpp)
     ldblk;
 
   int XorMe = 0;
+  int c;
 
   x=0;
   y=0;
@@ -494,10 +511,11 @@ static int UnpackWPG2Raster(Image *image,int bpp)
   UpImgBuff=MagickAllocateMemory(unsigned char *,(size_t) ldblk);
   if(UpImgBuff==NULL)
   {
-    MagickFreeMemory(BImgBuff);
+    FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
     return(-2);
   }
   (void) memset(UpImgBuff,0,ldblk);
+  (void) memset(SampleBuffer,0,sizeof(SampleBuffer));
 
   while( y< image->rows)
     {
@@ -506,11 +524,22 @@ static int UnpackWPG2Raster(Image *image,int bpp)
       switch(bbuf)
         {
         case 0x7D:
-          SampleSize=ReadBlobByte(image);  /* DSZ */
+          if ((c = ReadBlobByte(image)) == EOF) /* DSZ */
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
+          SampleSize=c;
           if(SampleSize>8)
-            return(-2);
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-2);
+            }
           if(SampleSize<1)
-            return(-2);
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-2);
+            }
           break;
         case 0x7E:
           if(y==0)			   /* XOR */
@@ -518,25 +547,41 @@ static int UnpackWPG2Raster(Image *image,int bpp)
 	  XorMe=!XorMe; /* or XorMe=1 ?? */
           break;
         case 0x7F:
-          RunCount=ReadBlobByte(image);   /* BLK */
+          if ((c = ReadBlobByte(image)) == EOF) /* BLK */
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
+          RunCount=c;
           for(i=0; i < SampleSize*(RunCount+1); i++)
             {
               InsertByte6(0);
             }
           break;
         case 0xFD:
-	  RunCount=ReadBlobByte(image);   /* EXT */
+          if ((c = ReadBlobByte(image)) == EOF) /* EXT */
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
+	  RunCount=c;
 	  for(i=0; i<= RunCount;i++)
             for(bbuf=0; bbuf < SampleSize; bbuf++)
-              InsertByte6(SampleBuffer[bbuf]);          
+              InsertByte6(SampleBuffer[bbuf]);
           break;
         case 0xFE:
-          RunCount=ReadBlobByte(image);  /* RST */
+          if ((c = ReadBlobByte(image)) == EOF)  /* RST */
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
+          RunCount = c;
           if(x!=0)
             {
               (void) fprintf(stderr,
                              "\nUnsupported WPG2 unaligned token RST x=%lu, please report!\n"
                              ,x);
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
               return(-3);
             }
           {
@@ -550,7 +595,12 @@ static int UnpackWPG2Raster(Image *image,int bpp)
           }
           break;
         case 0xFF:
-          RunCount=ReadBlobByte(image);	 /* WHT */
+          if ((c = ReadBlobByte(image)) == EOF)	 /* WHT */
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
+          RunCount=c;
           for(i=0; i < SampleSize*(RunCount+1); i++)
             {
               InsertByte6(0xFF);
@@ -574,10 +624,14 @@ static int UnpackWPG2Raster(Image *image,int bpp)
                 InsertByte6(bbuf);
               }
           }
+          if (EOFBlob(image))
+            {
+              FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
+              return(-4);
+            }
         }
     }
-  MagickFreeMemory(BImgBuff);
-  MagickFreeMemory(UpImgBuff);
+  FreeUnpackWPG2RasterAllocs(BImgBuff,UpImgBuff);
   return(0);
 }
 
@@ -597,17 +651,17 @@ unsigned Flags;
  (*CTM)[2][2]=1;
 
  Flags=ReadBlobLSBShort(image);
- if(Flags & LCK) x=ReadBlobLSBLong(image);	/*Edit lock*/
+ if(Flags & LCK) /*x=*/ (void) ReadBlobLSBLong(image);	/*Edit lock*/
  if(Flags & OID)
 	{
 	if(Precision==0)
-	  {x=ReadBlobLSBShort(image);}	/*ObjectID*/
+	  {/*x=*/ (void) ReadBlobLSBShort(image);}	/*ObjectID*/
 	else
-	  {x=ReadBlobLSBLong(image);}	/*ObjectID (Double precision)*/
+	  {/*x=*/ (void) ReadBlobLSBLong(image);}	/*ObjectID (Double precision)*/
 	}
  if(Flags & ROT)
 	{
-	x=ReadBlobLSBLong(image);	/*Rot Angle*/
+        x=ReadBlobLSBLong(image);	/*Rot Angle*/
 	if(Angle) *Angle=x/65536.0;
 	}
  if(Flags & (ROT|SCL))
@@ -832,11 +886,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     unsigned int NumOfEntries;
   } WPGColorMapRec;
 
+  /*
   typedef struct {
     unsigned long PS_unknown1;
     unsigned int PS_unknown2;
     unsigned int PS_unknown3;
-  } WPGPSl1Record;  
+  } WPGPSl1Record;
+  */
 
   Image
     *image,
@@ -1062,13 +1118,12 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     }
                 }      
 
-              if(UnpackWPGRaster(image,bpp) < 0)
-                /* The raster cannot be unpacked */
-                {
+			  if(!image_info->ping)
+                if(UnpackWPGRaster(image,bpp) < 0)                
+                {		/* The raster cannot be unpacked */
                 DecompressionFailed:
-                  ThrowReaderException(CoderError,UnableToDecompressImage,
-                                       image)
-                    }
+                  ThrowReaderException(CoderError,UnableToDecompressImage,image)
+                }
 
               if(Rec.RecType==0x14 && BitmapHeader2.RotAngle!=0 && !image_info->ping)
                 {  
@@ -1076,29 +1131,40 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                   if(BitmapHeader2.RotAngle & 0x8000)
                     {
                       rotated_image = FlopImage(image, exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);
+                        }
                     }
                   /* flip command */
                   if(BitmapHeader2.RotAngle & 0x2000)
                     {
                       rotated_image = FlipImage(image, exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);		
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);		
+                        }
                     }
 		
 		  /* rotate command */
                   if(BitmapHeader2.RotAngle & 0x0FFF)
                     {
-                      rotated_image = RotateImage(image, (BitmapHeader2.RotAngle & 0x0FFF), exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);		
+                      rotated_image = RotateImage(image,
+                                                  (BitmapHeader2.RotAngle & 0x0FFF),
+                                                  exception);
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);
+                        }
                     }                
                 }
 
@@ -1246,10 +1312,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(CTM[0][0]<0 && !image_info->ping)
 		{		/*?? RotAngle=360-RotAngle;*/
 		  rotated_image = FlopImage(image, exception);
-		  rotated_image->blob = image->blob;
-		  image->blob = NULL;
-		  (void) RemoveLastImageFromList(&image);
-		  AppendImageToList(&image,rotated_image);
+                  if (rotated_image != (Image *) NULL)
+                    {
+                      rotated_image->blob = image->blob;
+                      image->blob = NULL;
+                      (void) RemoveLastImageFromList(&image);
+                      AppendImageToList(&image,rotated_image);
+                    }
                   /* Try to change CTM according to Flip - I am not sure, must be checked.		  
                      Tx(0,0)=-1;      Tx(1,0)=0;   Tx(2,0)=0;
                      Tx(0,1)= 0;      Tx(1,1)=1;   Tx(2,1)=0;
@@ -1259,10 +1328,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(CTM[1][1]<0 && !image_info->ping)
 		{		/*?? RotAngle=360-RotAngle;*/
 		  rotated_image = FlipImage(image, exception);
-		  rotated_image->blob = image->blob;
-		  image->blob = NULL;
-		  (void) RemoveLastImageFromList(&image);
-		  AppendImageToList(&image,rotated_image);
+                  if (rotated_image != (Image *) NULL)
+                    {
+                      rotated_image->blob = image->blob;
+                      image->blob = NULL;
+                      (void) RemoveLastImageFromList(&image);
+                      AppendImageToList(&image,rotated_image);
+                    }
                   /* Try to change CTM according to Flip - I am not sure, must be checked.
                      float_matrix Tx(3,3);
                      Tx(0,0)= 1;   Tx(1,0)= 0;   Tx(2,0)=0;
