@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2009 GraphicsMagick Group
+% Copyright (C) 2003 - 2016 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -61,7 +61,8 @@ typedef enum
   XMLFileOutput = 0x0004,
   TXTFileOutput = 0x0008,
   Win32DebugOutput = 0x0010,
-  Win32EventlogOutput = 0x0020
+  Win32EventlogOutput = 0x0020,
+  MethodOutput = 0x0040
 } LogOutputType;
 
 typedef struct _OutputInfo
@@ -72,21 +73,15 @@ typedef struct _OutputInfo
 
 typedef struct _LogInfo
 {
-  char
-    *path,
-    *filename;
-
-  unsigned long
-    generations,
-    limit;
-
-  char
-    *format;
-
   FILE
     *file;
 
+  TimerInfo
+    timer;
+
   unsigned long
+    generations,
+    limit,
     generation,
     count;
 
@@ -96,8 +91,14 @@ typedef struct _LogInfo
   LogOutputType
     output_type;
 
-  TimerInfo
-    timer;
+  LogMethod
+    method;  /* Logger callback function */
+
+  char
+    path[256],
+    filename[256],
+    format[200];
+
 } LogInfo;
 
 typedef struct _EventInfo
@@ -172,7 +173,7 @@ static volatile unsigned int
   log_configured = False;
 
 static LogInfo
-  *log_info = (LogInfo *) NULL;
+  log_info;
 
 static SemaphoreInfo
   *log_semaphore = (SemaphoreInfo *) NULL;
@@ -183,37 +184,6 @@ static SemaphoreInfo
 static MagickPassFail
   ReadLogConfigureFile(const char *,const unsigned int,ExceptionInfo *);
 
-/*
-  Allocate LogInfo structure populated with default values
-*/
-static void AllocateLogInfo( void )
-{
-  LockSemaphoreInfo(log_semaphore);
-  if (log_info == (LogInfo *) NULL)
-    {
-      log_info=MagickAllocateMemory(LogInfo *,sizeof(LogInfo));
-      if (log_info == (LogInfo *) NULL)
-        MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-                          UnableToAllocateLogInfo);
-      (void) memset((void *) log_info,0,sizeof(LogInfo));
-      log_info->path=AcquireString("(default)");
-      log_info->filename=AcquireString("Magick-%d.log");
-      log_info->generations=3;
-      log_info->limit=2000;
-      log_info->format=AcquireString("%t %r %u %p %m/%f/%l/%d:\n  %e");
-      log_info->file=(FILE *) NULL;
-      log_info->generation=0;
-      log_info->count=0;
-      log_info->events=NoEventsMask;
-#if defined(MSWINDOWS)
-      log_info->output_type=Win32EventlogOutput;
-#else
-      log_info->output_type=StderrOutput;
-#endif
-      GetTimerInfo(&log_info->timer);
-    }
-  UnlockSemaphoreInfo(log_semaphore);
-}
 /*
   Parse an event specification string and return the equivalent bits.
 */
@@ -268,21 +238,20 @@ static LogEventType ParseEvents(const char *event_string)
 */
 MagickExport void DestroyLogInfo(void)
 {
-  if (log_info != (LogInfo *) NULL)
-    {
-      if (log_info->file != (FILE *) NULL)
-        if ((log_info->file != stdout) && (log_info->file != stderr))
-          {
-            (void) fprintf(log_info->file,"</log>\n");
-            (void) fclose(log_info->file);
-          }
-      MagickFreeMemory(log_info->filename);
-      MagickFreeMemory(log_info->path);
-      MagickFreeMemory(log_info->format);
-      MagickFreeMemory(log_info);
-    }
-  log_info=(LogInfo *) NULL;
+
+  LockSemaphoreInfo(log_semaphore);
+
+  if (log_info.file != (FILE *) NULL)
+    if ((log_info.file != stdout) && (log_info.file != stderr))
+      {
+        (void) fprintf(log_info.file,"</log>\n");
+        (void) fclose(log_info.file);
+        log_info.file=(FILE *) NULL;
+      }
   log_configured=False;
+
+  UnlockSemaphoreInfo(log_semaphore);
+
   DestroySemaphoreInfo(&log_semaphore);
 }
 
@@ -297,7 +266,11 @@ MagickExport void DestroyLogInfo(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method InitializeLogInfo initializes the constitute facility
+%  Method InitializeLogInfo initializes the logging facility.  This function
+%  is invoked by InitializeMagick(), which must be invoked before using any
+%  other APIs.  A memory-allocation failure in this function results in a
+%  fatal error for the whole program.  All of the allocations performed by
+%  this function are released by DestroyLogInfo().
 %
 %  The format of the InitializeLogInfo method is:
 %
@@ -308,8 +281,125 @@ MagickExport void DestroyLogInfo(void)
 MagickPassFail
 InitializeLogInfo(void)
 {
+  const char
+    *p;
+
+  /*
+    Initialize logging semaphore
+  */
   assert(log_semaphore == (SemaphoreInfo *) NULL);
   log_semaphore=AllocateSemaphoreInfo();
+
+  /*
+    Initialize LogInfo
+  */
+  (void) memset(&log_info,0,sizeof(LogInfo));
+
+  /*
+    Lock for access (to make Coverity happy)
+  */
+  LockSemaphoreInfo(log_semaphore);
+
+  (void) strlcpy(log_info.path,"(default)",sizeof(log_info.path));
+  (void) strlcpy(log_info.filename,"Magick-%d.log",sizeof(log_info.filename));
+  log_info.generations=3;
+  log_info.limit=2000;
+  (void) strlcpy(log_info.format,"%t %r %u %p %m/%f/%l/%d:\n  %e",
+                 sizeof(log_info.format));
+  log_info.file=(FILE *) NULL;
+  log_info.generation=0;
+  log_info.count=0;
+  log_info.events=NoEventsMask;
+#if defined(MSWINDOWS)
+  log_info.output_type=Win32EventlogOutput;
+#else
+  log_info.output_type=StderrOutput;
+#endif
+  GetTimerInfo(&log_info.timer);
+
+  UnlockSemaphoreInfo(log_semaphore);
+
+  /*
+    Set initial logging flags using the value of MAGICK_DEBUG if it is
+    set in the environment.  We do this here so it is possible to
+    debug the loading of the log configuration file.
+  */
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
+
+#if UseInstalledMagick
+  /*
+    Try to read the log configuration file.
+  */
+  {
+    ExceptionInfo
+      exception;
+
+    GetExceptionInfo(&exception);
+    (void) ReadLogConfigureFile(MagickLogFilename,0,&exception);
+    DestroyExceptionInfo(&exception);
+  }
+
+  /*
+    Set override logging flags using the value of MAGICK_DEBUG if it
+    is set in the environment.
+  */
+  if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+    (void) SetLogEventMask(p);
+#endif /* UseInstalledMagick */
+
+  return MagickPass;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   I n i t i a l i z e L o g I n f o P o s t                                 %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Method InitializeLogInfoPost finishes initialization of the logging
+%  facility after the installation path is known.  This function
+%  is invoked by InitializeMagick(), which must be invoked before using any
+%  other APIs.  All of the allocations performed by this function are
+%  released by DestroyLogInfo().
+%
+%  The format of the InitializeLogInfoPost method is:
+%
+%      MagickPassFail InitializeLogInfoPost(void)
+%
+%
+*/
+MagickPassFail
+InitializeLogInfoPost(void)
+{
+  const char
+    *p;
+
+  /*
+    Try to read the log configuration file.
+  */
+  if (!log_configured)
+  {
+    ExceptionInfo
+      exception;
+
+    GetExceptionInfo(&exception);
+    (void) ReadLogConfigureFile(MagickLogFilename,0,&exception);
+    DestroyExceptionInfo(&exception);
+
+    /*
+      Set override logging flags using the value of MAGICK_DEBUG if it
+      is set in the environment.
+    */
+    if ((p=getenv("MAGICK_DEBUG")) != (const char *) NULL)
+      (void) SetLogEventMask(p);
+  }
+
   return MagickPass;
 }
 
@@ -335,7 +425,7 @@ InitializeLogInfo(void)
 */
 MagickExport MagickBool IsEventLogging(void)
 {
-  return ((log_info) && (log_info->events != NoEventsMask));
+  return (log_info.events != NoEventsMask);
 }
 
 /*
@@ -401,7 +491,7 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
   if (!IsEventLogging())
     return(False);
 
-  if (log_info->events != AllEventsMask)
+  if (log_info.events != AllEventsMask)
     {
       unsigned int
         i;
@@ -422,7 +512,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if (((int) type >= eventmask_map[i].start_type) &&
                   ((int) type <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info->events) & ((unsigned int) eventmask_map[i].mask))
+                  if (((unsigned int) log_info.events) &
+                      ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
                       break;
@@ -437,7 +528,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
               if ((((int) type % 100) >= eventmask_map[i].start_type) &&
                   (((int) type % 100) <= eventmask_map[i].end_type))
                 {
-                  if (((unsigned int) log_info->events) & ((unsigned int) eventmask_map[i].mask))
+                  if (((unsigned int) log_info.events) &
+                      ((unsigned int) eventmask_map[i].mask))
                     {
                       enabled=True;
                       break;
@@ -517,115 +609,121 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
 #endif
   seconds=time((time_t *) NULL);
   time_meridian=localtime(&seconds);
-  elapsed_time=GetElapsedTime(&log_info->timer);
-  user_time=GetUserTime(&log_info->timer);
-  (void) ContinueTimer((TimerInfo *) &log_info->timer);
+  elapsed_time=GetElapsedTime(&log_info.timer);
+  user_time=GetUserTime(&log_info.timer);
+  (void) ContinueTimer((TimerInfo *) &log_info.timer);
   FormatString(timestamp,"%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+
     1900,time_meridian->tm_mon+1,time_meridian->tm_mday,
     time_meridian->tm_hour,time_meridian->tm_min,time_meridian->tm_sec);
-  if (((unsigned int) log_info->output_type) & XMLFileOutput)
+  if (((unsigned int) log_info.output_type) & XMLFileOutput)
     {
       /*
         Log to a file in the XML format.
       */
-      log_info->count++;
-      if (log_info->count >= log_info->limit)
+      log_info.count++;
+      if (log_info.count >= log_info.limit)
         {
-          (void) fprintf(log_info->file,"</log>\n");
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-          log_info->count=0;
+          (void) fprintf(log_info.file,"</log>\n");
+          (void) fclose(log_info.file);
+          log_info.file=(FILE *) NULL;
+          log_info.count=0;
         }
-      if (log_info->file == (FILE *) NULL)
+      if (log_info.file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
-                                     MagickFalse,log_info->generation);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
+                                     MagickFalse,log_info.generation);
+          log_info.file=fopen(filename,"w");
+          if (log_info.file == (FILE *) NULL)
             {
               UnlockSemaphoreInfo(log_semaphore);
               return(False);
             }
-          (void) fprintf(log_info->file,"<?xml version=\"1.0\"?>\n");
-          (void) fprintf(log_info->file,"<log>\n");
-          log_info->generation++;
-          if (log_info->generation >= log_info->generations)
-            log_info->generation=0;
+          (void) fprintf(log_info.file,"<?xml version=\"1.0\"?>\n");
+          (void) fprintf(log_info.file,"<log>\n");
+          log_info.generation++;
+          if (log_info.generation >= log_info.generations)
+            log_info.generation=0;
         }
-      (void) fprintf(log_info->file,"<record>\n");
-      (void) fprintf(log_info->file,"  <timestamp>%.1024s</timestamp>\n",
+      (void) fprintf(log_info.file,"<record>\n");
+      (void) fprintf(log_info.file,"  <timestamp>%.1024s</timestamp>\n",
         timestamp);
-      (void) fprintf(log_info->file,
+      (void) fprintf(log_info.file,
         "  <elapsed-time>%ld:%02ld</elapsed-time>\n",
         (long) (elapsed_time/60.0),(long) ceil(fmod(elapsed_time,60.0)));
-      (void) fprintf(log_info->file,"  <user-time>%0.3f</user-time>\n",
+      (void) fprintf(log_info.file,"  <user-time>%0.3f</user-time>\n",
         user_time);
-      (void) fprintf(log_info->file,"  <pid>%ld</pid>\n",(long) getpid());
-      (void) fprintf(log_info->file,"  <module>%.1024s</module>\n",srcname);
-      (void) fprintf(log_info->file,"  <function>%.1024s</function>\n",
+      (void) fprintf(log_info.file,"  <pid>%ld</pid>\n",(long) getpid());
+      (void) fprintf(log_info.file,"  <module>%.1024s</module>\n",srcname);
+      (void) fprintf(log_info.file,"  <function>%.1024s</function>\n",
         function);
-      (void) fprintf(log_info->file,"  <line>%lu</line>\n",line);
-      (void) fprintf(log_info->file,"  <domain>%.1024s</domain>\n",domain);
-      (void) fprintf(log_info->file,"  <severity>%.1024s</severity>\n",severity);
-      (void) fprintf(log_info->file,"  <event>%.1024s</event>\n",event);
-      (void) fprintf(log_info->file,"</record>\n");
-      (void) fflush(log_info->file);
+      (void) fprintf(log_info.file,"  <line>%lu</line>\n",line);
+      (void) fprintf(log_info.file,"  <domain>%.1024s</domain>\n",domain);
+      (void) fprintf(log_info.file,"  <severity>%.1024s</severity>\n",severity);
+      (void) fprintf(log_info.file,"  <event>%.1024s</event>\n",event);
+      (void) fprintf(log_info.file,"</record>\n");
+      (void) fflush(log_info.file);
       UnlockSemaphoreInfo(log_semaphore);
       return(True);
     }
-  if (((unsigned int) log_info->output_type) & TXTFileOutput)
+  if (((unsigned int) log_info.output_type) & TXTFileOutput)
     {
       /*
         Log to a file in the TXT format.
       */
-      log_info->count++;
-      if (log_info->count >= log_info->limit)
+      log_info.count++;
+      if (log_info.count >= log_info.limit)
         {
-          (void) fclose(log_info->file);
-          log_info->file=(FILE *) NULL;
-          log_info->count=0;
+          (void) fclose(log_info.file);
+          log_info.file=(FILE *) NULL;
+          log_info.count=0;
         }
-      if (log_info->file == (FILE *) NULL)
+      if (log_info.file == (FILE *) NULL)
         {
           char
             filename[MaxTextExtent];
 
-          (void) MagickSceneFileName(filename,log_info->filename,".%lu",
-                                     MagickFalse,log_info->generation);
-          log_info->file=fopen(filename,"w");
-          if (log_info->file == (FILE *) NULL)
+          (void) MagickSceneFileName(filename,log_info.filename,".%lu",
+                                     MagickFalse,log_info.generation);
+          log_info.file=fopen(filename,"w");
+          if (log_info.file == (FILE *) NULL)
             {
               UnlockSemaphoreInfo(log_semaphore);
               return(False);
             }
-          log_info->generation++;
-          if (log_info->generation >= log_info->generations)
-            log_info->generation=0;
+          log_info.generation++;
+          if (log_info.generation >= log_info.generations)
+            log_info.generation=0;
         }
-        (void) fprintf(log_info->file,
-          "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s %.1024s %.1024s\n",
-            timestamp, (long) (elapsed_time/60.0), (long) ceil(fmod(elapsed_time,60.0)),
-              user_time, (long) getpid(), srcname, function, line, domain, severity, event);
-      (void) fflush(log_info->file);
+        (void) fprintf(log_info.file,
+                       "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu"
+                       " %.1024s %.1024s %.1024s\n",
+                       timestamp, (long) (elapsed_time/60.0),
+                       (long) ceil(fmod(elapsed_time,60.0)),
+                       user_time, (long) getpid(), srcname, function, line, domain,
+                       severity, event);
+      (void) fflush(log_info.file);
       UnlockSemaphoreInfo(log_semaphore);
       return(True);
     }
 #if defined(MSWINDOWS)
-  if (log_info->output_type & Win32DebugOutput)
+  if (log_info.output_type & Win32DebugOutput)
     {
       char
         buffer[MaxTextExtent];
 
       FormatString(buffer,
-        "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s %.1024s %.1024s\n",
-          timestamp, (long) (elapsed_time/60.0), (long) ceil(fmod(elapsed_time,60.0)),
-            user_time, (long) getpid(), srcname, function, line, domain, severity, event);
+                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   " %.1024s %.1024s\n",
+                   timestamp, (long) (elapsed_time/60.0),
+                   (long) ceil(fmod(elapsed_time,60.0)),
+                   user_time, (long) getpid(), srcname, function, line,
+                   domain, severity, event);
       OutputDebugString(buffer);
     }
-  if (log_info->output_type & Win32EventlogOutput)
+  if (log_info.output_type & Win32EventlogOutput)
     {
 #define LOGGING_ERROR_CODE 0
       char
@@ -638,9 +736,12 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         hSource;
 
       FormatString(buffer,
-        "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s %.1024s %.1024s\n",
-          timestamp, (long) (elapsed_time/60.0), (long) ceil(fmod(elapsed_time,60.0)),
-            user_time, (long) getpid(), srcname, function, line, domain, severity, event);
+                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   " %.1024s %.1024s\n",
+                   timestamp, (long) (elapsed_time/60.0),
+                   (long) ceil(fmod(elapsed_time,60.0)),
+                   user_time, (long) getpid(), srcname, function, line,
+                   domain, severity, event);
       hSource = RegisterEventSource(NULL, MagickPackageName);
       if (hSource != NULL)
         {
@@ -650,8 +751,8 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         }
     }
 #endif
-  if ((((unsigned int) log_info->output_type) & StdoutOutput) ||
-      (((unsigned int) log_info->output_type) & StderrOutput))
+  if ((((unsigned int) log_info.output_type) & StdoutOutput) ||
+      (((unsigned int) log_info.output_type) & StderrOutput))
     {
       FILE
         *file;
@@ -659,9 +760,9 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
         Log to stdout in a "human readable" format.
       */
       file = stdout;
-      if (((unsigned int) log_info->output_type) & StderrOutput)
+      if (((unsigned int) log_info.output_type) & StderrOutput)
         file = stderr;
-      for (p=log_info->format; *p != '\0'; p++)
+      for (p=log_info.format; *p != '\0'; p++)
       {
         /*
           Process formatting characters in text.
@@ -758,6 +859,21 @@ MagickExport  unsigned int LogMagickEventList(const ExceptionType type,
       (void) fprintf(file,"\n");
       (void) fflush(file);
     }
+  if ((log_info.output_type & MethodOutput) &&
+      (log_info.method != (LogMethod) NULL))
+    {
+      char
+        buffer[MaxTextExtent];
+
+      FormatString(buffer,
+                   "%.1024s %ld:%02ld %0.3f %ld %.1024s %.1024s %lu %.1024s"
+                   " %.1024s %.1024s\n",
+                   timestamp, (long) (elapsed_time/60.0),
+                   (long) ceil(fmod(elapsed_time,60.0)),
+                   user_time, (long) getpid(), srcname, function, line,
+                   domain, severity, event);
+      log_info.method(type,buffer);
+    }
   UnlockSemaphoreInfo(log_semaphore);
   return(True);
 }
@@ -820,7 +936,8 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
     *xml;
 
   size_t
-    length=0;
+    length=0,
+    token_max_length;
 
   MagickPassFail
     status=MagickPass;
@@ -828,7 +945,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
   /*
     Read the log configure file.
   */
-  (void) strcpy(path,basename);
+  (void) strlcpy(path,basename,sizeof(path));
   if (depth == 0)
     {
       /*
@@ -847,12 +964,13 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
   if (xml == (char *) NULL)
     return MagickFail;
   token=AllocateString(xml);
+  token_max_length=strlen(token);
   for (q=xml; *q != '\0'; )
   {
     /*
       Interpret Coder.
     */
-    GetToken(q,&q,token);
+    MagickGetToken(q,&q,token,token_max_length);
     if (*token == '\0')
       break;
     (void) strlcpy(keyword,token,MaxTextExtent);
@@ -862,7 +980,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -873,10 +991,10 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
         while ((*token != '>') && (*q != '\0'))
         {
           (void) strlcpy(keyword,token,MaxTextExtent);
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
           if (*token != '=')
             continue;
-          GetToken(q,&q,token);
+          MagickGetToken(q,&q,token,token_max_length);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
@@ -905,26 +1023,21 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       }
     if (LocaleCompare(keyword,"<magicklog>") == 0)
       {
-        if (log_info == (LogInfo *) NULL)
-          MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-            UnableToAllocateLogInfo);
-        (void) CloneString(&log_info->path,path);
+        (void) strlcpy(log_info.path,path,sizeof(log_info.path));
         continue;
       }
-    if (log_info == (LogInfo *) NULL)
-      continue;
-    GetToken(q,(char **) NULL,token);
+    MagickGetToken(q,(char **) NULL,token,token_max_length);
     if (*token != '=')
       continue;
-    GetToken(q,&q,token);
-    GetToken(q,&q,token);
+    MagickGetToken(q,&q,token,token_max_length);
+    MagickGetToken(q,&q,token,token_max_length);
     switch (*keyword)
     {
       case 'E':
       case 'e':
       {
         if (LocaleCompare((char *) keyword,"events") == 0)
-          log_info->events = (LogEventType) ((unsigned int) log_info->events |
+          log_info.events = (LogEventType) ((unsigned int) log_info.events |
                                              (unsigned int) ParseEvents(token));
         break;
       }
@@ -933,12 +1046,12 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"filename") == 0)
           {
-            (void) CloneString(&log_info->filename,token);
+            (void) strlcpy(log_info.filename,token,sizeof(log_info.filename));
             break;
           }
         if (LocaleCompare((char *) keyword,"format") == 0)
           {
-            (void) CloneString(&log_info->format,token);
+            (void) strlcpy(log_info.format,token,sizeof(log_info.format));
             break;
           }
         break;
@@ -948,7 +1061,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"generations") == 0)
           {
-            log_info->generations=MagickAtoL(token);
+            log_info.generations=MagickAtoL(token);
             break;
           }
         break;
@@ -958,7 +1071,7 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
       {
         if (LocaleCompare((char *) keyword,"limit") == 0)
           {
-            log_info->limit=MagickAtoL(token);
+            log_info.limit=MagickAtoL(token);
             break;
           }
         break;
@@ -972,13 +1085,22 @@ static MagickPassFail ReadLogConfigureFile(const char *basename,
 
             for (i=0; output_map[i].name != 0; i++)
             {
-              if (LocaleNCompare(token,output_map[i].name,strlen(output_map[i].name)) == 0)
+              if (LocaleNCompare(token,output_map[i].name,
+                                 strlen(output_map[i].name)) == 0)
                 {
-                  /* We do not OR these flags despite the fact that they are bit masks
-                     because they are still mutually exclusive implementations. Asking for
-                     XML and TXT format files each use the file handle field and others to
-                     do their work, so they can not be used together */
-                  log_info->output_type=output_map[i].mask;
+                  /* We do not OR these flags despite the fact that
+                     they are bit masks because they are still
+                     mutually exclusive implementations. Asking for
+                     XML and TXT format files each use the file handle
+                     field and others to do their work, so they can
+                     not be used together */
+
+                  LockSemaphoreInfo(log_semaphore);
+
+                  log_info.output_type=output_map[i].mask;
+
+                  UnlockSemaphoreInfo(log_semaphore);
+
                   break;
                 }
             }
@@ -1029,38 +1151,21 @@ MagickExport unsigned long SetLogEventMask(const char *events)
   LogEventType
     event_flags=NoEventsMask;
 
-  if (log_info == (LogInfo *) NULL)
-    AllocateLogInfo();
-
   LockSemaphoreInfo(log_semaphore);
 
   if (events != NULL)
     {
       event_flags=ParseEvents(events);
-      log_info->events=event_flags;
+      log_info.events=event_flags;
     }
 
-  if (log_configured == False)
-    {
-      ExceptionInfo
-        exception;
+  event_flags=log_info.events;
 
-      UnlockSemaphoreInfo(log_semaphore);
-      GetExceptionInfo(&exception);
-      (void) ReadLogConfigureFile(MagickLogFilename,0,&exception);
-      DestroyExceptionInfo(&exception);
-      LockSemaphoreInfo(log_semaphore);
-    }
-
-  /*
-    If events were specified, then override configuration file value.
-  */
-  if (events != NULL)
-    log_info->events=event_flags;
-
-  event_flags=log_info->events;
   UnlockSemaphoreInfo(log_semaphore);
 
+  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                        "Set log event mask: %s", events ? events : "None");
+
   return (event_flags);
 }
 
@@ -1090,21 +1195,45 @@ MagickExport unsigned long SetLogEventMask(const char *events)
 */
 MagickExport void SetLogFormat(const char *format)
 {
-  if (log_info == (LogInfo *) NULL)
-    AllocateLogInfo();
-
   LockSemaphoreInfo(log_semaphore);
 
-  if (log_configured == False)
-    {
-      ExceptionInfo
-        exception;
+  (void) strlcpy(log_info.format,format,sizeof(log_info.format));
+
+  UnlockSemaphoreInfo(log_semaphore);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S e t L o g M e t h o d                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SetLogMethod() sets the method that should be called when logging.
+%
+%  The format of the SetLogMethod method is:
+%
+%      SetLogFormat(LogMethod)
+%
+%  A description of each parameter follows:
+%
+%    o method: pointer to a method that will be called when LogMagickEvent is
+%      being called.
+%
+%
+*/
+MagickExport void SetLogMethod(LogMethod method)
+{
+  LockSemaphoreInfo(log_semaphore);
+
+  log_info.output_type=(LogOutputType) (log_info.output_type |
+    MethodOutput);
+  log_info.method=method;
 
-      GetExceptionInfo(&exception);
-      (void) ReadLogConfigureFile(MagickLogFilename,0,&exception);
-      DestroyExceptionInfo(&exception);
-    }
-
-  (void) CloneString(&log_info->format,format);
   UnlockSemaphoreInfo(log_semaphore);
 }
