@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003 - 2013 GraphicsMagick Group
+  Copyright (C) 2003 - 2016 GraphicsMagick Group
   Copyright (C) 2002 ImageMagick Studio
  
   This program is covered by multiple licenses, which are described in
@@ -23,8 +23,9 @@ extern "C" {
 
 /*
   Note that the WIN32 and WIN64 definitions are provided by the build
-  configuration rather than the compiler.  Definitions available from
-  the Windows compiler are _WIN32 and _WIN64.
+  configuration rather than the compiler.  Definitions available from the
+  Windows compiler are _WIN32 and _WIN64.  Note that _WIN32 is defined if
+  _WIN64 is defined.
 */
 #if defined(WIN32) || defined(WIN64)
 #  define MSWINDOWS
@@ -51,12 +52,14 @@ extern "C" {
   threads.  Most CPUs have cache lines of 32 or 64 bytes.  IBM Power CPUs have
   cache lines of 128 bytes.
 */
-#if defined(MAGICK_TARGET_CPU) && (MAGICK_TARGET_CPU == powerpc)
+/* C pre-processor does not support comparing strings. */
+#if defined(__powerpc__)
 #  define MAGICK_CACHE_LINE_SIZE 128
 #else
 #  define MAGICK_CACHE_LINE_SIZE 64
 #endif
 
+
 /*
   Support library symbol prefixing
 */
@@ -138,6 +141,10 @@ extern "C" {
 #  include <sys/timeb.h>
 #endif
 
+#if defined(HAVE_STDINT_H)
+#  include <stdint.h>
+#endif
+
 #if defined(POSIX)
 #  if defined(HAVE_SYS_NDIR_H) || defined(HAVE_SYS_DIR_H) || defined(HAVE_NDIR_H)
 #    define dirent direct
@@ -311,6 +318,7 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  define ProcessPendingEvents(text)
 #  define ReadCommandlLine(argc,argv)
 #  define SetNotifyHandlers
+#  define MagickSleep(seconds) sleep(seconds)
 #endif
 
 #if defined(MSWINDOWS)
@@ -326,8 +334,7 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  define SetNotifyHandlers \
     SetErrorHandler(NTErrorHandler); \
     SetWarningHandler(NTWarningHandler)
-#  undef sleep
-#  define sleep(seconds)  Sleep(seconds*1000)
+#  define MagickSleep(seconds)  Sleep(seconds*1000)
 #  if !defined(HAVE_TIFFCONF_H)
 #    define HAVE_TIFFCONF_H
 #  endif
@@ -344,6 +351,9 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #define MagickEpsilon  1.0e-12
 #define MagickPI  3.14159265358979323846264338327950288419716939937510
 #define MagickSQ2PI 2.50662827463100024161235523934010416269302368164062
+#if !defined(INFINITY) /* C'99 provides INFINITY but C'89 does not */
+#  define INFINITY (log(0))
+#endif
 #define Max(x,y)  (((x) > (y)) ? (x) : (y))
 #define Min(x,y)  (((x) < (y)) ? (x) : (y))
 #define NumberOfObjectsInArray(octets,size) ((octets+size-1)/size)
@@ -411,40 +421,6 @@ extern int vsnprintf(char *s, size_t n, const char *format, va_list ap);
 #  define BuildMagickModules
 #endif
 
-
-/*
-  I/O defines.
-*/
-#if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__)
-  /* Windows '95 and Borland C do not support _lseeki64 */
-#  define MagickSeek(fildes,offset,whence)  _lseeki64(fildes,/* __int64 */ offset,whence)
-#  define MagickTell(fildes) /* __int64 */ _telli64(fildes)
-#else
-#  define MagickSeek(fildes,offset,whence)  lseek(fildes,offset,whence)
-#  define MagickTell(fildes) (MagickSeek(fildes,0,SEEK_CUR))
-#endif
-
-#if defined(MSWINDOWS) && !defined(Windows95) && !defined(__BORLANDC__) && \
-  !(defined(_MSC_VER) && _MSC_VER < 1400) && \
-  !(defined(__MINGW32__) && __MSVCRT_VERSION__ < 0x800)
-  /*
-    Windows '95 and Borland C do not support _lseeki64
-    Visual Studio does not support _fseeki64 and _ftelli64 until the 2005 release.
-    Without these interfaces, files over 2GB in size are not supported for Windows.
-  */
-#  define MagickFseek(stream,offset,whence) _fseeki64(stream,/* __int64 */ offset,whence)
-#  define MagickFstat(fildes,stat_buff) _fstati64(fildes,/* struct _stati64 */ stat_buff)
-#  define MagickFtell(stream) /* __int64 */ _ftelli64(stream)
-#  define MagickStatStruct_t struct _stati64
-#  define MagickStat(path,stat_buff) _stati64(path,/* struct _stati64 */ stat_buff)
-#else
-#  define MagickFseek(stream,offset,whence) fseek(stream,offset,whence)
-#  define MagickFstat(fildes,stat_buff) fstat(fildes,stat_buff)
-#  define MagickFtell(stream) ftell(stream)
-#  define MagickStatStruct_t struct stat
-#  define MagickStat(path,stat_buff) stat(path,stat_buff)
-#endif
-
 /*
   C99 isblank() is not portable enough yet.
 */
